/*****************************************************************************
 *  Copyright (C) 2002  Jason Surprise <thesurprises1@attbi.com>             *
 *                2003  Enno Bartels   <ennobartels@t-online.de>             *
 *****************************************************************************
 *                                                                           *
 *  This program is free software; you can redistribute it and/or            *
 *  modify it under the terms of the GNU Library General Public              *
 *  License as published by the Free Software Foundation; either             *
 *  version 2 of the License, or (at your option) any later version.         *
 *                                                                           *
 *  This program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
 *  Library General Public License for more details.                         *
 *                                                                           *
 *  You should have received a copy of the GNU Library General Public        *
 *  License along with this program; see the file COPYING.LIB.  If not,      *
 *  write to the Free Software Foundation, Inc., 59 Temple Place -           *
 *  Suite 330,  Boston, MA 02111-1307, USA.                                  *
 *                                                                           *
 *****************************************************************************
 *                                                                           *
 *  Project name.....  hp215                                                 *
 *                                                                           *
 *  Modul name.......  hp215.h                                               *
 *                                                                           *
 *  Starting dates...  11.Nov.2002 (Enno Bartels)                            *
 *                     --.---.2002 (Jason Surprise)                          *
 *                                                                           *
 *  Version..........  0.2.3                                                 *
 *                                                                           *
 *  Author...........  - Options & autoconf/-make framework written & new    *
 *                       maintainer: Enno Bartels <ennobartels@t-online.de>  *
 *                     - Checksum code written by Roberto Ragusa             *
 *                     - Jason Surprise <thesurprises1@attbi.com>            *
 *                                                                           *
 *  Description......  Linux USB Driver for the HP Photosmart 215 Camera     *
 *                                                                           *
 *****************************************************************************/


#ifndef _HP215_H
#define _HP215_H

/* Possible modes */
#define NORMAL_MODE 1  /* Normal mode               */
#define PRTCMD_MODE 2  /* Print camera command mode */
#define TSTDAT_MODE 3  /* Test date string  mode    */

/* Element types on the date string option */
#define DATE  1 
#define ASCII 2

#define MAX_ITEMS 200


#define PREVIEW_DIR  "preview"
#define FULLSIZE_DIR "fullsize"

/* Function call answers*/
#define SUCCESS 0
#define FAIL    1

/* Mode of the getDateStr function */
#define MUST_USE_A_PICTURE_NUMBER      1
#define MUST_NOT_USE_A_PICTURE_NUMBER  2

/* Maxlength of the format string */
#define MAX_DS_STRLEN 1000

/* Maximal pictures*/
#define MAX_PICS 1000

/* ACK hexcode */
#define ACK 0x06

#define HP_VENDOR_ID 0x3f0
#define HP215_ID     0x6202

/* General commands */
#define HP_CMD_INIT            {0x02, 0xce, 0x80, 0x8a, 0x84, 0x8d, 0x83, 0x03}
#define HP_CMD_DATETIME        {0x02, 0xc1, 0x80, 0x8b, 0x84, 0x8e, 0x8d, 0x03}
#define HP_CMD_GET_PHOTO_ALBUM {0x02, 0xc6, 0x88, 0x80, 0x80, 0x83, 0x84, 0x38, 0x2f, 0x30, 0x32, 0x88, 0x84, 0x8e, 0x8b, 0x03}
#define HP_CMD_DELETE_PICS     {0x02, 0xb1, 0x84, 0x8f, 0x8f, 0x8f, 0x8f, 0x86, 0x80, 0x8a, 0x86, 0x03}

/* Enum for the date/time elements on the date string option */
enum myCodes 
{  
   N_02,        /* N_ = Number */
   N_03,
   N_02_PLUS1,
   N_03_PLUS1,

   SUM_PICS_02, /* Number of pictures */
   SUM_PICS_03, 

   D_02,        /* D_ = Day of month */

   M_LONG,      /* M_ = Months */
   M_SHORT,
   M_02,       
   M_,       

   Y_02,        /* Y_ = Years */
   Y_04,

   H_02_12,     /* H_ = Hours */
   H_02_24,
   H____12,     
   H____24,
   H_AMPM,

   K_02,        /* K_ = Minutes */
};



typedef struct 
{
  unsigned int hour;
  unsigned int min;
  unsigned int sec;

  unsigned int day;
  unsigned int month;
  unsigned int year;

} t_date;



typedef struct  
{
    char              name [1000];
    char              name_nsd [1000];

    char              preview_name [1000];
    char              preview_name_nsd [1000];


    unsigned long int size;
    unsigned char    *data;
    t_date            date; 
    char              date_time [20];
} CamFile;


typedef struct  
{
    struct usb_device     *device;
    struct usb_dev_handle *dh;
    int                    num_pics;
    CamFile               *pic[MAX_PICS];
} t_camera;




typedef struct
{
    /* Main mode */
    int main;           /* Was a mode selected (0=No;else yes) */

    /* Selected modes */
    int dateCam;        /* Mode: Get date of the camera ?    */
    int albm;           /* Mode: Get number of pictures ?    */
    int datePics;       /* Mode: Get date/time of pictures ? */
    int prev;           /* Mode: Get previews ?              */
    int pics;           /* Mode: Get pictures ?              */
    int delt;           /* Mode: Delete pictures ?           */
    int html;           /* Mode: Create a html page ?        */
    int tmp_html;

    /* Options */
    int  tryInitCamTwice;      /* Try cam init 2 times ?            */
    int  forceDeletion;        /* Do deletion without questions ?   */
    int  picNumbersAll;        /* Where all pictures selected ?     */
    int  picNumbers;           /* Number of selected pictures       */
    int  picNumber[MAX_PICS];  /* Selected pictures                 */  

    int  picNumbersRL;          /* Number of selected pictures      */
    int  picNumberRL[MAX_PICS]; /* Selected pictures                */ 
    int  picNumbersRR;          /* Number of selected pictures      */
    int  picNumberRR[MAX_PICS]; /* Selected pictures                */ 
    int  picNumbersRH;          /* Number of selected pictures      */
    int  picNumberRH[MAX_PICS]; /* Selected pictures                */ 
 
    char subDir[100];           /* Name of the sub directory        */
    int  subDirPrev;            /* A preview sub directory ?        */
    int  subDirFull;            /* A fullsize sub directory ?       */

    /*  char subDirPrev[100];  */    
    /*  char subDirFull[100];  */     /* Name of the preview sub directory */

    /* Options for the specital mode print command */
    int  printCmd_cmd;         /* Command code */
    int  printCmd_picNo;       /* Picture number */

    char dateStr[MAX_DS_STRLEN];

    int  dateInPic;             /* Date string inside the picture   */
    char dateInPicStr[MAX_DS_STRLEN]; /* Date string format inside pic.   */
} t_mode;


/* Function prototypes */
void  dprint              (char    *);
struct usb_device *
      find_device         (int, int);
int   hp_open             (t_camera *, int);
int   hp_init_sequence    (t_camera *);

int   hp_new_file         (CamFile **);
int   hp_append_file      (CamFile *, const unsigned char *, unsigned long int);

int   hp_sent_ack         (t_camera *, int);
int   hp_rcv_ack          (t_camera *, int);

int   hp_gen_cmd          (int, int, unsigned char *);
int   hp_get_timeDate_cam (t_camera *);
int   hp_get_photo_album  (t_camera *);
int   hp_get_timeDate_pics(t_camera *, int, int [MAX_PICS], char *, char *, int, int);
int   hp_get_previews     (t_camera *, int, int [MAX_PICS], char *);
int   hp_write_previews   (t_camera *, int, int [MAX_PICS]);
int   hp_get_write_pics   (t_camera *, int, int [MAX_PICS], char *);
int   hp_delete_pics      (t_camera *, int, int, int[MAX_PICS]);
int   hp_delete_all_pics  (t_camera *);
int   hp_delete_some_pics (t_camera *, int, int[MAX_PICS]);
void  hp_create_html_page (t_camera *, int, int[MAX_PICS], char *, int, int, int);

int   hp_close            (t_camera *);

void  read_ini            (t_mode *mode);
void  read_cmd            (int *, char ***, t_mode *);
void  usage               (void);
void  gen_subDirName      (char [100]);
void  gen_subDir          (char [100]);
void  read_number         (int *, int [1000], char *);
void  getDateStr          (char *, unsigned int, unsigned int, t_date, char *,
                           int);
void  test_dateString     (char *);
void  check_dateStr       (char *); 
void  test_cmd            (t_mode);
void  ini_getNoElements   (int *, char *);
void  ini_delComment      (char *);
void  acceptOptions       (t_mode *, int *, char **[], int);
void  init_modeValues     (t_mode *);
void  checkPid            (void);
void  myExit              (int, int);
void  removeLockFile      (void);
void  add_dateInsidePicture (char     *dateInPicStr,
                            int       picNumbers, 
                            int       picNumber[MAX_PICS],
			     t_camera *myCam);
#endif
